/*
 *  ClipPlayer.java
 *
 *  This file is part of the Java Sound Examples.
 */

/*
 *  Copyright (c) 1999 - 2001 by Matthias Pfisterer <Matthias.Pfisterer@web.de>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as published
 *   by the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


import  java.io.File;
import  java.io.IOException;

import  javax.sound.sampled.AudioFormat;
import  javax.sound.sampled.AudioInputStream;
import  javax.sound.sampled.AudioSystem;
import  javax.sound.sampled.Clip;
import  javax.sound.sampled.DataLine;
import  javax.sound.sampled.LineEvent;
import  javax.sound.sampled.LineListener;
import  javax.sound.sampled.LineUnavailableException;



/*  +DocBookXML
    <title>Playing an audio file multiple times (Clip)</title>

    <formalpara><title>Purpose</title>
    <para>
    Plays a single audio file multiple times in sequence.
    Demonstrates the usage of Java Sound's
    <classname>Clip</classname>.
    </para></formalpara>

    <formalpara><title>Level</title>
    <para>newbie</para></formalpara>

    <formalpara><title>Usage</title>
    <cmdsynopsis>
    <command>java ClipPlayer</command>
    <arg choice="plain"><replaceable>audiofile</replaceable></arg>
    <arg choice="plain"><replaceable>#loops</replaceable></arg>
    </cmdsynopsis>
    </formalpara>

    <formalpara><title>Parameters</title>
    <variablelist>
    <varlistentry>
    <term><option><replaceable>audiofile</replaceable></option></term>
    <listitem><para>the name of the
    audio file that should be played</para></listitem>
    </varlistentry>
    <varlistentry>
    <term><option><replaceable>#loops</replaceable></option></term>
    <listitem><para>how often the file should be
    repeated (0 means play once)</para></listitem>
    </varlistentry>
    </variablelist>
    </formalpara>

    <formalpara><title>Bugs, limitations</title>
    <para>Only PCM encoded files are supported. A-law, &mu;-law,
    ADPCM, mp3, GSM and other compressed data formats are
    not supported. For a technique of handling these see
    AudioPlayer.</para>
    </formalpara>

    <formalpara><title>Source code</title>
    <para>
    <ulink url="ClipPlayer.java.html">ClipPlayer.java</ulink>
    </para></formalpara>

-DocBookXML
*/
 class ClipPlayer
    implements  LineListener
{
    private Clip        m_clip;


    /*
     *  The clip will be played nLoopCount + 1 times.
     */
    public ClipPlayer(File clipFile, int nLoopCount)
    {
        AudioInputStream    audioInputStream = null;
        try
        {
            audioInputStream = AudioSystem.getAudioInputStream(clipFile);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        if (audioInputStream != null)
        {
            AudioFormat format = audioInputStream.getFormat();
            DataLine.Info   info = new DataLine.Info(Clip.class, format);
            try
            {
                m_clip = (Clip) AudioSystem.getLine(info);
                m_clip.addLineListener(this);
                m_clip.open(audioInputStream);
            }
            catch (LineUnavailableException e)
            {
                e.printStackTrace();
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
            m_clip.loop(nLoopCount);
        }
        else
        {
            System.out.println("ClipPlayer.<init>(): can't get data from file " + clipFile.getName());
        }
    }



    public void update(LineEvent event)
    {
        if (event.getType().equals(LineEvent.Type.STOP))
        {
            m_clip.close();
        }
        else if (event.getType().equals(LineEvent.Type.CLOSE))
        {
            /*
             *  There is a bug in the jdk1.3.0.
             *  It prevents correct termination of the VM.
             *  So we have to exit ourselves.
             */
            System.exit(0);
        }

    }



    public static void main(String[] args)
    {
        if (args.length != 2)
        {
            System.out.println("ClipPlayer: usage:");
            System.out.println("\tjava ClipPlayer <soundfile> <#loops>");
        }
        else
        {
            File    clipFile = new File(args[0]);
            int     nLoopCount = Integer.parseInt(args[1]);
            ClipPlayer  clipPlayer = new ClipPlayer(clipFile, nLoopCount);
        }
    }
}



/*** ClipPlayer.java ***/

